@@ -77,6 +77,10 @@ all_opt = {
 		"getopt" : "",
 		"help" : "",
 		"order" : 1},
+	"force_on" : {
+		"getopt" : "",
+		"help" : "",
+		"order" : 1},
 	"action" : {
 		"getopt" : "o:",
 		"longopt" : "action",
@@ -124,10 +128,22 @@ all_opt = {
 		"getopt" : "",
 		"help" : "",
 		"order" : 1},
+	"no_on" : {
+		"getopt" : "",
+		"help" : "",
+		"order" : 1},
+	"no_off" : {
+		"getopt" : "",
+		"help" : "",
+		"order" : 1},
 	"telnet" : {
 		"getopt" : "",
 		"help" : "",
 		"order" : ""},
+	"diag" : {
+		"getopt" : "",
+		"help" : "",
+		"order" : ""},
 	"passwd" : {
 		"getopt" : "p:",
 		"longopt" : "password",
@@ -476,7 +492,9 @@ def _add_dependency_options(options):
 		if DEPENDENCY_OPT.has_key(opt):
 			added_opt.extend([y for y in DEPENDENCY_OPT[opt] if options.count(y) == 0])
 
-	if not "port" in (options + added_opt) and not "nodename" in (options + added_opt):
+	if not "port" in (options + added_opt) and \
+			not "nodename" in (options + added_opt) and \
+			"ipaddr" in (options + added_opt):
 		added_opt.append("port_as_ip")
 		all_opt["port"]["help"] = "-n, --plug=[ip]                IP address or hostname of fencing device " \
 			"(together with --port-as-ip)"
@@ -571,9 +589,8 @@ def metadata(avail_opt, docs):
 
 	if "on" in available_actions:
 		available_actions.remove("on")
-
-	on_target = ' on_target="1"' if avail_opt.count("on_target") else ''
-	print "\t<action name=\"on\"%s automatic=\"%d\"/>" % (on_target, avail_opt.count("fabric_fencing"))
+		on_target = ' on_target="1"' if avail_opt.count("on_target") else ''
+		print "\t<action name=\"on\"%s automatic=\"%d\"/>" % (on_target, avail_opt.count("fabric_fencing"))
 
 	for action in available_actions:
 		print "\t<action name=\"%s\" />" % (action)
@@ -749,7 +766,8 @@ def fence_action(connection, options, set_power_fn, get_power_fn, get_outlet_lis
 		## Process options that manipulate fencing device
 		#####
 		if (options["--action"] in ["list", "list-status"]) or \
-			((options["--action"] == "monitor") and 1 == options["device_opt"].count("port")):
+			((options["--action"] == "monitor") and 1 == options["device_opt"].count("port") and \
+			0 == options["device_opt"].count("port_as_ip")):
 
 			if 0 == options["device_opt"].count("port"):
 				print "N/A"
@@ -768,9 +786,9 @@ def fence_action(connection, options, set_power_fn, get_power_fn, get_outlet_lis
 				## keys can be numbers (port numbers) or strings (names of VM, UUID)
 				for outlet_id in outlets.keys():
 					(alias, status) = outlets[outlet_id]
-					status = status.upper()
-					if not status in ["ON", "OFF"]:
+					if status is None or (not status.upper() in ["ON", "OFF"]):
 						status = "UNKNOWN"
+						status = status.upper()
 
 					if options["--action"] == "list":
 						print outlet_id + options["--separator"] + alias
@@ -779,14 +797,20 @@ def fence_action(connection, options, set_power_fn, get_power_fn, get_outlet_lis
 
 			return
 
-		status = get_multi_power_fn(connection, options, get_power_fn)
+		if options["--action"] == "monitor" and not "port" in options["device_opt"] and "no_status" in options["device_opt"]:
+			# Unable to do standard monitoring because 'status' action is not available
+			return 0
 
-		if status != "on" and status != "off":
-			fail(EC_STATUS)
+		status = None
+		if not "no_status" in options["device_opt"]:
+			status = get_multi_power_fn(connection, options, get_power_fn)
+			if status != "on" and status != "off":
+				fail(EC_STATUS)
 
 		if options["--action"] == status:
-			print "Success: Already %s" % (status.upper())
-			return 0
+			if not (status == "on" and "force_on" in options["device_opt"]):
+				print "Success: Already %s" % (status.upper())
+				return 0
 
 		if options["--action"] == "on":
 			if set_multi_power_fn(connection, options, set_power_fn, get_power_fn, 1 + int(options["--retry-on"])):
@@ -887,7 +911,11 @@ def run_command(options, command, timeout=None, env=None):
 	if timeout is not None:
 		timeout = float(timeout)
 
-	logging.info("Executing: %s\n", command)
+	# For IPMI password occurs on command line, it should not be part of debug info
+	log_command = command
+	if "ipmitool" in log_command:
+		log_command = re.sub("-P (.+?) -p", "-P [set] -p", log_command)
+	logging.info("Executing: %s\n", log_command)
 
 	try:
 		process = subprocess.Popen(shlex.split(command), stdout=subprocess.PIPE, stderr=subprocess.PIPE, env=env)
@@ -1030,7 +1058,7 @@ def _login_telnet(options, re_login_string):
 		options["eol"] = "\n"
 		conn.send_eol(options["--username"])
 		conn.log_expect(re_pass, int(options["--login-timeout"]))
-	elif re_pass.search(screen) != None:
+	elif re_pass.search(screen) == None:
 		conn.log_expect(re_pass, int(options["--shell-timeout"]))
 
 	try:
@@ -1043,7 +1071,7 @@ def _login_telnet(options, re_login_string):
 			conn.send_eol("")
 			screen = conn.read_nonblocking(size=100, timeout=int(options["--shell-timeout"]))
 			## after sending EOL the fence device can either show 'Login' or 'Password'
-			if re_login.search(screen) != None:
+			if re_login.search(conn.after + screen) != None:
 				conn.send_eol("")
 			conn.send_eol(options["--username"])
 			conn.log_expect(re_pass, int(options["--login-timeout"]))
@@ -1107,6 +1135,10 @@ def _update_metadata(options):
 	else:
 		all_opt["login"]["required"] = "0"
 
+	if device_opt.count("port_as_ip"):
+		all_opt["ipaddr"]["required"] = "0"
+		all_opt["port"]["required"] = "0"
+
 	(available_actions, default_value) = _get_available_actions(device_opt)
 	all_opt["action"]["default"] = default_value
 
@@ -1175,9 +1207,6 @@ def _validate_input(options, stop = True):
 	device_opt = options["device_opt"]
 	valid_input = True
 
-	if "port_as_ip" in device_opt and not "--port-as-ip" in options and "--plug" in options:
-		fail_usage("Parser error: option -n/--plug is not recognized")
-
 	if not options.has_key("--username") and \
 			device_opt.count("login") and (device_opt.count("no_login") == 0):
 		valid_input = False
@@ -1277,7 +1306,9 @@ def _parse_input_cmdline(avail_opt):
 	(getopt_string, longopt_list) = _prepare_getopt_args(avail_opt)
 
 	try:
-		entered_opt = getopt.gnu_getopt(sys.argv[1:], getopt_string, longopt_list)[0]
+		(entered_opt, left_arg) = getopt.gnu_getopt(sys.argv[1:], getopt_string, longopt_list)
+		if len(left_arg) > 0:
+			logging.warning("Unused arguments on command line: %s" % (str(left_arg)))
 	except getopt.GetoptError, error:
 		fail_usage("Parse error: " + error.msg)
 
@@ -1291,6 +1322,10 @@ def _parse_input_cmdline(avail_opt):
 			if "--" + value.get("longopt", "") == arg_name or "-" + value.get("getopt", "").rstrip(":") == arg_name][0]
 		long_opts["--" + filtered_opts[all_key]["longopt"]] = dict(entered_opt)[arg_name]
 
+	# This test is specific because it does not apply to input on stdin
+	if "port_as_ip" in avail_opt and not "--port-as-ip" in long_opts and "--plug" in long_opts:
+		fail_usage("Parser error: option -n/--plug is not recognized")
+
 	return long_opts
 
 # for ["John", "Mary", "Eli"] returns "John, Mary and Eli"
@@ -1347,8 +1382,14 @@ def _get_available_actions(device_opt):
 		default_value = "off"
 	if device_opt.count("no_status"):
 		available_actions.remove("status")
+	if device_opt.count("no_on"):
+		available_actions.remove("on")
+	if device_opt.count("no_off"):
+		available_actions.remove("off")
 	if not device_opt.count("separator"):
 		available_actions.remove("list")
 		available_actions.remove("list-status")
+	if device_opt.count("diag"):
+		available_actions.append("diag")
 
 	return (available_actions, default_value)
